/*
*  Touchscreen controlled marionnette
*
*  We are going to have a little fun with the Galileo. 
*  This time around, we��ll make a simple puppet control system. 
*
*  You'll need a Adafruit TFT Touch screen,
*  a servo motor, a LED and 300 Ohm resistor
*  for this example.
*
*  (c) 2015 Arduino LLC
*
*/
#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#include <Servo.h>

#include <stdint.h>
#include "TouchScreen.h"

#include "./util.h"

#define YP A2  // must be an analog pin, use "An" notation!
#define XM A3  // must be an analog pin, use "An" notation!
#define YM 8   // can be a digital pin
#define XP 9   // can be a digital pin

#define TFT_DC 6
#define TFT_CS 5

//Declare the TFT screen, and touch layer
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);

//Declare the UI components
Slider sld;
Button btn;
Checkbox chb;

//Declare servo
Servo myservo;

//Tracking UI state
boolean btnVal_old;
boolean onOff;


void setup() {
  Serial.begin(9600);
  
  //Initialize UI
  tft.begin();
  attach(&tft,&ts);
  
  setRotation(1);
  tft.fillScreen(BLK);
  calibrate();
  
  //Set the origin position of UI elements
  sld.begin(55,45);
  btn.begin(125,45);
  chb.begin(125,145);
  
  //Initialize servo and LED
  myservo.attach(10);
  pinMode(7,OUTPUT);
}

void loop() {
  struct Vector val=getTouchPoint();
  
  //Update the UI components
  sld.draw(val);
  btn.draw(val);
  chb.draw(val);
  
  //servo speed control
  if(!onOff){
    myservo.write(90);
  }else{
    myservo.write(sld.value/100.0*180);
  }
  
  //start or stop servo
  if(btn.value==true && btnVal_old==false){
    onOff=!onOff;
  }
  btnVal_old=btn.value;
  
  //LED
  digitalWrite(7,chb.value);
}
