#include "./util.h"
struct Vector minPair,maxPair;

Adafruit_ILI9341* _tft;
TouchScreen* _ts;

int _rotation;

void attach(Adafruit_ILI9341* tft2, TouchScreen* ts2){
  _tft=tft2;
  _ts=ts2;
}

void setRotation(int rotation){
  //TFT->setRotation only rotates the display, 
  //using _rotation to calculate the touch point accordingly
  _rotation=rotation;
  _tft->setRotation(_rotation);
}

struct Vector getCalibPair(){
  //Get a vector in calibration process
  int pressure;
  struct Vector res;
  int counter=0;
  while(true){
    pressure=_ts->pressure();
    if(pressure>_ts->pressureThreshhold && pressure<700){
      TSPoint p = _ts->getPoint();
      res.x=p.x;
      res.y=p.y;
      Serial.print(res.x);
      Serial.print(" ");
      Serial.println(res.y);
      if(++counter>=3)break;
      
      //break;
    }
  }
  return res;
}
struct Vector getTouchPoint(){
  //Get the current touched point
  static struct Vector tp={WIDTH/2,HEIGHT/2};
  int pressure=_ts->pressure();
  if (pressure > _ts->pressureThreshhold && pressure < 700) {    
    TSPoint p = _ts->getPoint();
    if(_rotation==0){
      tp.x=map(p.x,minPair.x,maxPair.x,0,WIDTH);
      tp.y=map(p.y,minPair.y,maxPair.y,0,HEIGHT);
    }else if(_rotation==1){
      tp.x=map(p.y,minPair.x,maxPair.x,0,WIDTH);
      tp.y=map(p.x,minPair.y,maxPair.y,0,HEIGHT);      
    }
    tp.touching=true;
    /*
    Serial.print(p.x);
    Serial.print(" ");
    Serial.print(p.y);
    Serial.println(" ");*/
    //Serial.println(pressure);
  }else{
    tp.touching=false;
  }
  return tp;
}

void drawCalibCorner(int seq){
  //Draw a rectange during calibration process
  int pos[8]={10,10, WIDTH-30,10, 10,HEIGHT-30, WIDTH-30,HEIGHT-30};
  int seqL=seq-1;
  if(seqL>=0){
    _tft->fillRect(pos[seqL*2],pos[seqL*2+1],20,20,BLK);
  }
  delay(100);
  if(seq<4){
    _tft->fillRect(pos[seq*2],pos[seq*2+1],20,20,WHT);
  }
}
void calibrate(){
  //The entire calibration process
  struct Vector corners[4];
  _tft->setTextSize(3);
  _tft->setCursor(35,70);
  _tft->println("Calibrating...");
  _tft->setTextSize(2);
  _tft->setCursor(55,146);
  _tft->println("Press the squares");

  for(int i=0;i<4;i++){
    drawCalibCorner(i);
    corners[i]=getCalibPair();
  }
  drawCalibCorner(4);
  _tft->fillScreen(BLK);

  if(_rotation==0){
    minPair.x=(corners[0].x+corners[2].x)/2;
    minPair.y=(corners[0].y+corners[1].y)/2;
  
    maxPair.x=(corners[1].x+corners[3].x)/2;
    maxPair.y=(corners[2].y+corners[3].y)/2;
  }else if(_rotation==1){
    //minPair.x=(corners[0].x+corners[1].x)/2;
    //minPair.y=(corners[0].y+corners[2].y)/2;
    minPair.x=corners[0].y;
    minPair.y=corners[0].x;
    maxPair.x=corners[3].y;
    maxPair.y=corners[3].x;
    //maxPair.x=(corners[3].x+corners[2].x)/2;
    //maxPair.y=(corners[3].y+corners[1].y)/2;
  }
  
  Serial.print(minPair.x);
  Serial.print(" ");
  
  Serial.print(minPair.y);
  Serial.print("   ");
  
  Serial.print(maxPair.x);
  Serial.print(" ");
  Serial.print(maxPair.y);
  Serial.println(" ");
}


void UIElement::begin(int x, int y, int width, int height){
  //generic initialization function of a UI element
  pOrig.x=x;
  pOrig.y=y;
  this->width=width;
  this->height=height;
  
  paddingX=20;
  paddingY=20;
}

bool UIElement::within(Vector pos){
  //Tell if a vector is inside the UI element
  if(pos.x<pOrig.x-paddingX || pos.x>pOrig.x+width+paddingX ||
    pos.y<pOrig.y-paddingY || pos.y>pOrig.y+height+paddingY || !pos.touching){
      return false;
    }
  return true;
}


//Slider
void Slider::begin(int x, int y, int width, int height){
  UIElement::begin(x,y,width,height);
  value=50;
  
  _tft->drawRect(pOrig.x-2,pOrig.y-4,width+4,height+9,CYN);
  draw(value);
}

int Slider::getValue(Vector pos){
  //Get a value between 0 and 100 as represented by the slider
  int value=100-(pos.y-pOrig.y)*1.0/height*100;
  if(value<0)value=0;
  if(value>100)value=100;
  
  return value;
}
void Slider::draw(Vector pos){
  //redraw the slider based on a touch point
  if(!within(pos)){
    if(touchPos.touching && !pos.touching){
      _tft->fillRect(pOrig.x+width+5,pOrig.y,20,7,BLK);
      _tft->setTextColor(CYN);
      _tft->setCursor(pOrig.x+width+5,pOrig.y);
      _tft->setTextSize(1);
      _tft->println(value);
  
    }
  }else{
    int value=getValue(pos);
    
    draw(value);
  }
  touchPos=pos;
}
void Slider::draw(int value){
  //redraw the slider based on its value
  static int lastVal=0;
  this->value=value;
  
  int diffVal=value-lastVal;

  int lastHeight=(lastVal/100.0)*height;
  int deltaHeight=diffVal/100.0*height;

  if(diffVal>0){
    _tft->fillRect(pOrig.x,pOrig.y+height-lastHeight-deltaHeight-2,width,deltaHeight+5,CYN);
  }else if(diffVal<0){
    _tft->fillRect(pOrig.x,pOrig.y+height-lastHeight-2,width,-deltaHeight+5,BLK);
  }
  /*
  if(diffVal!=0){
    _tft->fillRect(pOrig.x+width+5,pOrig.y,20,7,BLK);
    _tft->setTextColor(CYN);
    _tft->setCursor(pOrig.x+width+5,pOrig.y);
    _tft->setTextSize(1);
    _tft->println(value);
  }*/
  lastVal=value;
}


//Button
void Button::begin(int x, int y, int width, int height){
  //Initialize the button UI element
  UIElement::begin(x,y,width,height);
  value=true;
  
  _tft->setTextColor(CYN);
  _tft->setCursor(pOrig.x+width+5,pOrig.y);
  _tft->setTextSize(2);
  _tft->print("START/");
  _tft->setCursor(pOrig.x+width+5,pOrig.y+20);
  _tft->print("STOP");

}

bool Button::getValue(Vector pos){
  //get a true/false value of the button
  if(within(pos))return true;
  else return false;
}
void Button::draw(Vector pos){
  //redraw the button based on the touch point
  bool value=getValue(pos);
  if(this->value!=value){
    if(value==true)
//      _tft->fillRect(pOrig.x,pOrig.y,width,height,WHT);
      _tft->fillRoundRect(pOrig.x,pOrig.y,width,height,5,WHT);
    else
//      _tft->fillRect(pOrig.x,pOrig.y,width,height,CYN);
      _tft->fillRoundRect(pOrig.x,pOrig.y,width,height,5,CYN);
  }
  this->value=value;
}


//Checkbox
void Checkbox::begin(int x, int y, int width, int height){
  //initialize the checkbox element
  UIElement::begin(x,y,width,height);
  left=true;

  _tft->drawRect(pOrig.x,pOrig.y,width,height,CYN);
  _tft->setTextColor(CYN);
  _tft->setCursor(pOrig.x+width+5,pOrig.y);
  _tft->setTextSize(2);
  _tft->println("LED");
}

bool Checkbox::getValue(Vector pos){
  //get the value of checkbox based on the current touch position
  bool in=within(pos);
  if(left){
    if(in){
      value=!value;
      left=false;
    }
  }else{
    if(!in)
      left=true;
  }
  return value;
}

void Checkbox::draw(Vector pos){
  //redraw checkbox based on the current touch position
  bool in=within(pos);
  bool changed=false;
  if(left){
    if(in){
      value=!value;
      changed=true;
      left=false;
    }
  }else{
    if(!in)
      left=true;
  }
  if(changed){
    if(value){
      _tft->fillRect(pOrig.x+5,pOrig.y+5,width-10,height-10,CYN);
    }else{
      _tft->fillRect(pOrig.x+5,pOrig.y+5,width-10,height-10,BLK);
    }
  }
}
