#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#include "TouchScreen.h"

#define WIDTH 320
#define HEIGHT 240

#define BLK ILI9341_BLACK
#define WHT ILI9341_WHITE
#define CYN ILI9341_CYAN

//Struct containing a vector
struct Vector{
  float x,y;
  bool touching;
};

//connect the touchscreen instances with the library
void attach(Adafruit_ILI9341* tft2, TouchScreen* ts2);

//functions used for simplifying the initialziation process
struct Vector getCalibPair();
struct Vector getTouchPoint();
void drawCalibCorner(int seq);
void calibrate();
void setRotation(int rotation);

//Base class for all the UI elements
class UIElement{
  public:
    virtual void begin(int x, int y, int width, int height);
    virtual void draw(Vector pos)=0;
    bool within(Vector pos);
  protected:
    struct Vector pOrig;
    struct Vector touchPos;
    int paddingX;
    int paddingY;
    
    int width;
    int height;
};

//A slider UI element
class Slider : public UIElement{
  public:
    void begin(int x, int y, int width=10, int height=150) override;
    void draw(int value);
    void draw(Vector pos);
    int getValue(Vector pos);
    int value;
};

//A button UI element
class Button : public UIElement{
  public:
    void begin(int x, int y, int width=60, int height=60) override;
    bool getValue(Vector pos);
    void draw(Vector pos);
    bool value;

};

//A checkbox UI element
class Checkbox: public UIElement{
  public:
    void begin(int x, int y, int width=30, int height=30) override;
    bool getValue(Vector pos);
    void draw(Vector pos);
    bool value;
  private:
    bool left;
};
